@@ -1,16 +1,17 @@
 /*  INT 16h emulation v0.1     (C) 1999. A'rpi/ESP-team  */
 
-static int
-em_int16(struct vm86_regs *r){
-int ax=(r->eax & 0xffff);
-int ah=(ax>>8);
-#define SET_AX(x) regX(r->eax) = (x)
+#include "rulez.h"
+
+static int em_int16 ( struct vm86_regs *r )
+{
+  int ax=(r->eax & 0xffff);
+  int ah=(ax>>8);
 
   /* Wait for a key */
   if(ah==0 || ah==0x10){
     unsigned char c;
     read(0,&c,1);
-    if(c==27) SET_AX(0x011B); else SET_AX(c);  /* FIX IT! */
+    if(c==27) AX=0x011B; else AX=c;  /* FIX IT! */
     return 1;
   }
 
@@ -27,7 +28,7 @@ int ah=(ax>>8);
     if (select(fd + 1, &fds, 0, 0, &tv) > 0) {
       /* is something in buffer */
       r->eflags&=~0x40;
-      SET_AX(0);                  /* FIX IT!!! */
+      AX=0;                  /* FIX IT!!! */
     } else {
       /* buffer is empty */
       r->eflags|=0x40;
@@ -45,5 +46,3 @@ int ah=(ax>>8);
   return 0;
 }
 
-#undef SET_AX
-
