@@ -31,6 +31,7 @@ This software has NO WARRANTY.  Use it at your own risk.
 
 #include "lrmi.h"
 #include "rulez.h"
+#include "int21.h"
 
 #define REAL_MEM_BASE 	0x10000
 #define REAL_MEM_SIZE 	(0xA0000-REAL_MEM_BASE)
@@ -139,7 +140,7 @@ LRMI_init(void)
 	if (fd_mem == -1)
 		{
 		perror("open /dev/mem");
-		return 0;
+//		return 0;
 		}
 
 	m = mmap((void *)0, 0x502,
@@ -160,7 +161,7 @@ LRMI_init(void)
 		{
 		perror("mmap /dev/mem");
 		return 0;
-		}
+		} 
 
 
 	/*
@@ -175,6 +176,7 @@ LRMI_init(void)
 	 Allocate the return to 32 bit routine
 	*/
 	m = (void*)dosmem_alloc(2);
+	printf("dosmem_alloc %d\n",(int)m);
 
 	context.ret_seg = (unsigned int)m >> 4;
 	context.ret_off = (unsigned int)m & 0xf;
@@ -662,7 +664,6 @@ debug_info(int vret)
 	fputs("]\n", stderr);
 	}
 
-#include "int21.c"
 #include "int16.c"
 
 
@@ -698,99 +699,96 @@ void sighandler(int x){
 }
 
 
-static int
-run_vm86(void)
-	{
-	unsigned int vret;
-	unsigned int v;
+static int run_vm86( void )
+{
+  unsigned int vret;
+  unsigned int v;
 
-	signal(SIGTRAP,&sighandler);
+  signal(SIGTRAP,&sighandler);
 
-	while (1)
-	{
-		vret = lrmi_vm86(&context.vm);
+  while (1) {
+    vret = lrmi_vm86(&context.vm);
 
-		if (VM86_TYPE(vret) == VM86_INTx)
-			{
-			v = VM86_ARG(vret);
+    if (VM86_TYPE(vret) == VM86_INTx) {
+      v = VM86_ARG(vret);
 
-			if (v == RETURN_TO_32_INT) return 1;
+      if (v == RETURN_TO_32_INT) return 1;
 
-			if(v==0x21){    /* DOS function call */
-			  int i=em_int21(&context.vm.regs);
-			  if(i==-1) return 1;
-			  continue;
-			}
-			if(v==0x16){    /* Keyboard functions */
-			  int i=em_int16(&context.vm.regs);
-			  if(i==-1) return 1;
-			  continue;
-			}
-			
-			if(v==0x29){    /* Fast write char (AL=asciicode) */
-			  putchar(context.vm.regs.eax & 0xff);
- 			  continue;
-			}
+      if (v==0x21) {    /* DOS function call */
+        int i=em_int21(&context.vm.regs);
+	if(i==-1) return 1;
+	continue;
+      }
+      if (v==0x16) {    /* Keyboard functions */
+	int i=em_int16(&context.vm.regs);
+	if (i==-1) return 1;
+	continue;
+      }
+      if (v==0x29) {    /* Fast write char (AL=asciicode) */
+	putchar(context.vm.regs.eax & 0xff);
+	fflush(stdout);
+ 	continue;
+      }
+      if (v==0x33) {    /* mouse */
+        continue;
+      }
 
-			if(v==0x33){    /* mouse */
- 			  continue;
-			}
-
-			if(v>=0x20) printf("Interrupt: %02Xh  (ax=0x%04X)\n",v,context.vm.regs.eax & 0xffff);
+      if(v>=0x20) printf("Interrupt: %02Xh  (ax=0x%04X)\n",v,context.vm.regs.eax & 0xffff);
 			
 do_int:
-			pushw(context.vm.regs.eflags);
-			pushw(context.vm.regs.cs);
-			pushw(context.vm.regs.eip);
+      pushw(context.vm.regs.eflags);
+      pushw(context.vm.regs.cs);
+      pushw(context.vm.regs.eip);
 
-			context.vm.regs.cs = get_int_seg(v);
-			context.vm.regs.eip = get_int_off(v);
-			context.vm.regs.eflags &= ~(VIF_MASK | TF_MASK);
+      context.vm.regs.cs = get_int_seg(v);
+      context.vm.regs.eip = get_int_off(v);
+      context.vm.regs.eflags &= ~(VIF_MASK | TF_MASK);
 			
-			continue;
-			}
+      continue;
+    }  /* if VM86_INTx */
 
-		if (VM86_TYPE(vret) == VM86_SIGNAL) {
-		  if(trap_flag){
-		    trap_flag=0;
-		    signal(SIGTRAP,&sighandler); /* requires! ? */
-		    v=3;goto do_int;  // doesn't work for INT1
-//		    continue;
-		  }
-		  printf("!!! Signal received\n");
-		}
+    if (VM86_TYPE(vret) == VM86_SIGNAL) {
+      if (trap_flag) {
+        trap_flag=0;
+	signal(SIGTRAP,&sighandler); /* requires! ? */
+	v=3;
+	goto do_int;  // doesn't work for INT1
+//	continue;
+      }
+      printf("!!! Signal received\n");
+    } /* if VM86_SIGNAL */
 		
 //		printf("! type=%d  arg=%d\n",VM86_TYPE(vret),VM86_ARG(vret));
 
-		if (VM86_TYPE(vret) != VM86_UNKNOWN)
-			break;
+    if (VM86_TYPE(vret) != VM86_UNKNOWN) break;
 
-		if (!emulate()){
-		    byte *p=(byte *)((dword)context.vm.regs.cs << 4);
-	            p+=context.vm.regs.eip;
-		    if(p[0]==0xE4){
-		      /* IN AL,??h */
-		      printf("Skipped: IN AL,%Xh\n",p[1]);
-		      context.vm.regs.eip+=2;
-		      continue;
-		    }
-		    if(p[0]==0xE6){
-		      /* IN AL,??h */
-		      printf("Skipped: OUT %Xh,AL\n",p[1]);
-		      context.vm.regs.eip+=2;
-		      continue;
-		    }
-			printf("Warning: Can't emulate  CS:IP=%04X:%04X\n",
-      			      context.vm.regs.cs, context.vm.regs.eip );
+    if (!emulate()) {
+      byte *p=(byte *)((dword)context.vm.regs.cs << 4);
+      p+=context.vm.regs.eip;
+      if(p[0]==0xE4) {
+        /* IN AL,??h */
+	fprintf(stderr,"Skipped: IN AL,%Xh\n",p[1]);
+	context.vm.regs.eip+=2;
+	continue;
+      }
+      if(p[0]==0xE6) {
+        /* IN AL,??h */
+	printf("Skipped: OUT %Xh,AL\n",p[1]);
+	context.vm.regs.eip+=2;
+	continue;
+      }
+      fprintf(stderr,"Warning: Can't emulate  CS:IP=%04X:%04X\n",context.vm.regs.cs, context.vm.regs.eip );
+      fprintf(stderr,"%02X %02X %02X %02X\n",p[0],p[1],p[2],p[3]);
+      
 //			break;
-		}
+    } /* if (!emulate()) */
 			
-	}
+  }  /* while (1) */
 
-	debug_info(vret);
+  debug_info(vret);
 
-	return 0;
-	}
+  return 0;
+}
 
 
 
